#version 430

layout (local_size_x = 16, local_size_y = 16, local_size_z = 1) in;

layout(std430, binding = 0) buffer pos0 { vec4 position_In[]; };
layout(std430, binding = 1) buffer velo0 { vec4 velocity_In[]; };
layout(std430, binding = 2) buffer pos1 { vec4 position_Out[]; };
layout(std430, binding = 3) buffer velo1 { vec4 velocity_Out[]; };

layout (binding = 4, offset = 0) uniform atomic_uint count;

uniform uint lastCount;
uniform uint maxCount;
uniform float deltaT;
uniform uint spawnCount;
uniform float timetolive;
uniform vec3 originalPosition;

void addParticleToOutputList(vec3 pos, vec3 velo, float TTL){
	
	uint nr = atomicCounterIncrement(count);
	if (nr >= maxCount) {
		atomicCounterDecrement(count);
		return;
	}
	position_Out[nr] = vec4(pos, TTL);
	velocity_Out[nr] = vec4(velo, 0);
		
}

void main(){
	uint idx = gl_GlobalInvocationID.x + gl_GlobalInvocationID.y * gl_NumWorkGroups.x * gl_WorkGroupSize.x;
	if (idx >= lastCount){
		return;
	}

	vec3 velocity = velocity_In[idx].xyz;
	float TTL = position_In[idx].w - deltaT;
	vec3 pos = position_In[idx].xyz + velocity_In[idx].xyz * deltaT;

	if (TTL < 2) {
		pos = position_In[idx].xyz + vec3(-1 * velocity.x, 1, -1 * velocity.z) * deltaT;
	}


	if (TTL > 0) {
		addParticleToOutputList(pos, velocity, TTL);
		
	} else {
		pos.x = originalPosition.x;
		pos.y = originalPosition.y;
		pos.z = originalPosition.z;
		addParticleToOutputList(pos, velocity, timetolive);
	}

}